/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc. Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation version 2 of the License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/

package com.netscape.admin.dirserv.panel.replication;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import com.netscape.admin.dirserv.*;
import com.netscape.admin.dirserv.panel.*;
import com.netscape.management.client.*;
import com.netscape.management.client.console.*;
import com.netscape.management.client.util.*;
import netscape.ldap.*;
import netscape.ldap.util.RDN;

/**
 * Destination Panel for Replication Agreement
 *
 * @author  jpanchen
 * @version %I%, %G%
 * @date	 	12/13/97
 * @see     com.netscape.admin.dirserv.panel.replication
 */
public class AgreementDestinationPanel extends ReplicationBlankPanel
    implements ActionListener, ItemListener, ITabPanel, DocumentListener {

    /*==========================================================
     * constructors
     *==========================================================*/

    /**
     * public constructor
     * construction is delayed until selected.
     * @param owner resource object owner
     */
    public AgreementDestinationPanel(IAgreementPanel parent, int index) {
        super(parent, index);
        setTitle(_resource.getString("replication-destination-tab","label"));
        _monitor = (AgreementInfoPanel)parent.getInfoTab();
        _consoleInfo = _model.getConsoleInfo();
        _serverInfo = _model.getServerInfo();
        _frame = _model.getFrame();
        _dsInstances = new Hashtable();
        _icon = ReplicationTool.getImage("directory.gif");
        if(_agreement.getAgreementType() == ReplicationAgreement.AGREEMENT_TYPE_AD)
        {
        	_helpToken = "configuration-sync-connection-help";
        } else {
        	_helpToken = "configuration-replication-content-help";
        }
		_refreshWhenSelect = false;
    }

    /*==========================================================
     * public methods
     *==========================================================*/

    /**
     * Actual panel construction
     */
    public void init() {		
		GridBagConstraints gbc = new GridBagConstraints();		
		_myPanel.setLayout(new GridBagLayout());
        _myPanel.setBackground(getBackground());    

        //ssl check box
        _sslEncrypt = makeJCheckBox(_resource.getString(
							"replication-destination-sslEncrypt","label"));
        ReplicationTool.resetGBC(gbc);
        gbc.anchor = gbc.WEST;        
        gbc.gridwidth = gbc.REMAINDER;
        gbc.fill = gbc.NONE;
        gbc.weightx=1.0;        
        _myPanel.add(_sslEncrypt, gbc);

        /* add DSEntry to correctly update field 
           coloring and buttons enabling/disabling */
        _sslDSEntry = new DSEntryBoolean ("off", _sslEncrypt);
        setComponentTable(_sslEncrypt, _sslDSEntry);        

        //auth using label
        JLabel auth = makeJLabel(_resource.getString(
						     "replication-destination-authUsing","label"));		
        ReplicationTool.resetGBC(gbc);
        gbc.anchor = gbc.WEST;        
        gbc.gridwidth = gbc.REMAINDER;
        gbc.fill = gbc.NONE;
        gbc.weightx=1.0;        
        _myPanel.add(auth, gbc);
		
		// Auth mode panel
		JPanel authModePanel = new JPanel(new GridBagLayout());        
		auth.setLabelFor(authModePanel);
        ReplicationTool.resetGBC(gbc);
        gbc.gridwidth = gbc.REMAINDER;
        gbc.weightx=1.0;
        gbc.insets = new Insets(0,0,0,0);        
        _myPanel.add(authModePanel, gbc);

        //ssl auth radio button
        ButtonGroup authGroup = new ButtonGroup();
        _sslAuth = makeJRadioButton(_resource.getString(
							"replication-destination-sslClientAuth","label"));
        authGroup.add(_sslAuth);
        ReplicationTool.resetGBC(gbc);
        gbc.anchor = gbc.WEST;        
        gbc.gridwidth = gbc.REMAINDER;
        gbc.fill = gbc.NONE;
        gbc.weightx=1.0;        
        authModePanel.add(_sslAuth, gbc);

        /* add DSEntry to correctly update field 
           coloring and buttons enabling/disabling */
		_sslAuthDSEntry = new DSEntryBoolean ("off", _sslAuth);
		setComponentTable(_sslAuth, _sslAuthDSEntry);        

        //simple auth radio button
        _simpAuth = makeJRadioButton(_resource.getString(
							 "replication-destination-simpleAuth","label"));
        authGroup.add(_simpAuth);
        ReplicationTool.resetGBC(gbc);
		gbc.insets.top = 0;
        gbc.anchor = gbc.WEST;        
        gbc.gridwidth = gbc.REMAINDER;
        gbc.fill = gbc.NONE;
        gbc.weightx=1.0;        
        authModePanel.add(_simpAuth, gbc);

        /* add DSEntry to correctly update field 
           coloring and buttons enabling/disabling */
		_simpAuthDSEntry = new DSEntryBoolean ("on", _simpAuth);
		setComponentTable(_simpAuth, _simpAuthDSEntry);  

        //simp panel
        JPanel simpPanel = new JPanel(new GridBagLayout());        
        ReplicationTool.resetGBC(gbc);
        gbc.gridwidth = gbc.REMAINDER;
        gbc.weightx=1.0;
        gbc.insets = new Insets(0,0,0,0);        
        authModePanel.add(simpPanel, gbc);

        //bind as
        _bind = makeJLabel(_resource.getString(
					       "replication-destination-bindAs","label"));
        ReplicationTool.resetGBC(gbc);
        gbc.anchor = gbc.NORTHEAST;        
        gbc.fill = gbc.NONE;
		gbc.insets.left +=20;		
        simpPanel.add(_bind, gbc);

        _bindText = makeJTextField(10);
		_bind.setLabelFor(_bindText);
        ReplicationTool.resetGBC(gbc);
		gbc.insets.right = gbc.insets.left;
        gbc.anchor = gbc.NORTHWEST;
        gbc.gridwidth = gbc.REMAINDER;
		gbc.weightx = 1.0;  
	// tmp solution until server suport it
        simpPanel.add(_bindText, gbc);
	
        /* add DSEntry to correctly update field
           coloring and buttons enabling/disabling */
		_bindDSEntry = new bindDSEntry ("", _bindText, _bind);
		setComponentTable(_bindText, _bindDSEntry);
		
        //password
        _pwd = makeJLabel(_resource.getString(
											  "replication-destination-bindPwd","label"));
        ReplicationTool.resetGBC(gbc);
        gbc.anchor = gbc.NORTHEAST;
		gbc.insets.left +=20;
        gbc.fill = gbc.NONE;
        simpPanel.add(_pwd, gbc);

        _pwdText= makeJPasswordField(10);		
		_pwd.setLabelFor(_pwdText);
        ReplicationTool.resetGBC(gbc);
		gbc.insets.right = gbc.insets.left;
        gbc.anchor = gbc.NORTHWEST;
        gbc.gridwidth = gbc.REMAINDER;
		gbc.weightx = 1.0;
        simpPanel.add(_pwdText, gbc);
        
        /* add DSEntry to correctly update field
           coloring and buttons enabling/disabling */
		_pwdDSEntry = new DSEntryTextStrict ("", _pwdText, _pwd);
		setComponentTable(_pwdText, _pwdDSEntry);
        
        if(_agreement.getAgreementType() == ReplicationAgreement.AGREEMENT_TYPE_AD){
            //new Windows user Sync
            _newWinUserSync = makeJLabel(_resource.getString(
            "replication-destination-new-user-sync","label"));
            _newWinUserSync.setToolTipText(_resource.getString(
            "replication-destination-new-user-sync","ttip"));
            ReplicationTool.resetGBC(gbc);
            gbc.anchor = gbc.NORTHEAST;
            gbc.insets.left +=20;
            gbc.fill = gbc.NONE;
            simpPanel.add(_newWinUserSync, gbc);
            
            _newWinUserSyncCB= makeJCheckBox();
            _newWinUserSync.setLabelFor(_newWinUserSyncCB);
            ReplicationTool.resetGBC(gbc);
            gbc.insets.right = gbc.insets.left;
            gbc.anchor = gbc.NORTHWEST;
            gbc.gridwidth = gbc.REMAINDER;
            gbc.weightx = 1.0;
            simpPanel.add(_newWinUserSyncCB, gbc);
            
            _newUserDSEntry = new DSEntryBoolean ("off", _newWinUserSyncCB);
		setComponentTable(_newWinUserSyncCB, _newUserDSEntry); 
/*            _newWinUserSyncCB.setEnabled(false); // Do not allow editting of this post-creation.
            _newWinUserSyncCB.repaint(1);
            _newWinUserSync.setEnabled(false);
            _newWinUserSync.repaint(1);*/
            
            /******** new Windows group Sync ***********/
            _newWinGroupSync = makeJLabel(_resource.getString(
            "replication-destination-new-group-sync","label"));
            _newWinGroupSync.setToolTipText(_resource.getString(
            "replication-destination-new-group-sync","ttip"));
            ReplicationTool.resetGBC(gbc);
            gbc.anchor = gbc.NORTHEAST;
            gbc.insets.left +=20;
            gbc.fill = gbc.NONE;
            simpPanel.add(_newWinGroupSync, gbc);
            
            _newWinGroupSyncCB= makeJCheckBox();
            _newWinGroupSync.setLabelFor(_newWinGroupSyncCB);
            ReplicationTool.resetGBC(gbc);
            gbc.insets.right = gbc.insets.left;
            gbc.anchor = gbc.NORTHWEST;
            gbc.gridwidth = gbc.REMAINDER;
            gbc.weightx = 1.0;
            simpPanel.add(_newWinGroupSyncCB, gbc);
            
            _newGroupDSEntry = new DSEntryBoolean ("off", _newWinGroupSyncCB);
            setComponentTable(_newWinGroupSyncCB, _newGroupDSEntry); 
/*            _newWinGroupSyncCB.setEnabled(false); // Do not allow editting of this post-creation.
            _newWinGroupSyncCB.repaint(1);
            _newWinGroupSync.setEnabled(false);
            _newWinGroupSync.repaint(1);*/
                
        }

        /* add DSEntry to correctly update field
           coloring and buttons enabling/disabling */
		  

		addBottomGlue();
        
        initialize ();		
		enableFields();
                
                
    }

    private void enableFields(){	
	_sslEncrypt.setEnabled(false);
	_sslAuth.setEnabled(_sslEncrypt.isSelected());
	_simpAuth.setEnabled(_sslEncrypt.isSelected());
    }


  /**
     * Update on-screen data from Directory.
	 *
	 * Note: we overwrite the data that the user may have modified.  This is done in order to keep
	 * the coherency between the refresh behaviour of the different panels of the configuration tab.
     *
     **/
	public boolean refresh() {
            if(!refreshed){
                refreshed=true;
		resetCallback();
            }
            refreshed=false;
		return true;
	}

    //========= ACTIONLISTENER =================
    public void actionPerformed(ActionEvent e) {        
        if (e.getSource().equals(_sslAuth)) {
            if (_sslAuth.isSelected()) {
                //disable
                enableSimpleAuth (false);
            }
        }
        if (e.getSource().equals(_simpAuth)) {
            if (_simpAuth.isSelected()) {
                //enable
                enableSimpleAuth (true);
            }
        }
        if (e.getSource().equals(_sslEncrypt)) {
	    setDirtyFlag();
            if (_sslEncrypt.isSelected()) {
                //enable
                _sslAuth.setEnabled(true);

                _portAttr = ATTR_SSL_PORT;
            } else {
                //disable
                _sslAuth.setEnabled(false);
                //enable
                _simpAuth.setSelected(true);
                enableSimpleAuth (true);

                _portAttr = ATTR_PORT;
            }
        }
        super.actionPerformed (e);
    }

    static void showErrorDialog( JFrame frame, String err,
				 String section ) {
	DSUtil.showErrorDialog( frame, err, "",
				section, _resource );
    }

    //===== ITabPanel interface functions =========

    //specify entry verification rules here
    public boolean validateEntries() {

        /* all other validation is done every time
           field is changed using DSEntry mechanism */
	// CopieFrom doesn't exist ...
        return true;
    }

    public void getUpdateInfo( Object inf ) {
		AgreementWizardInfo info = (AgreementWizardInfo)inf;

        if (_sslEncrypt.isSelected())
            info.setSSL(true);
        else
            info.setSSL(false);

        if (!_sslAuth.isSelected()) {
            info.setBindDN(_bindText.getText());
            info.setBindPWD(_pwdText.getText());
	    info.setSSLAuth( false );
            if(_agreement.getAgreementType() == ReplicationAgreement.AGREEMENT_TYPE_AD)
            {
                if(_newWinUserSyncCB.isSelected())
                {
                    info.setNewWinUserSync("on");
                }else{
                    info.setNewWinUserSync("off");
                }
                if(_newWinGroupSyncCB.isSelected())
                {
                    info.setNewWinGroupSync("on");
                }else{
                    info.setNewWinGroupSync("off");
                }
            }
        } else {
            info.setBindDN("");
            info.setBindPWD("");
	    info.setSSLAuth( true );
        }
    }

    /*==========================================================
     * private methods
     *==========================================================*/

    /**
     * Populate the UI with data
     */
    protected void populateData() {

        /* populate all fileds except to and from */
        if ( (_agreement.getBindCredentials() == null) || (_agreement.getBindDN() == null) ) {
            _agreement.updateAgreementFromServer();
        }
        copyData ();
    }
 
    /**
     * Copy Rest of data from agreement
     */
    private void copyData() {
        if ((_agreement.getBindCredentials()!= null) &&
	    (_agreement.getBindDN()!= null)){
            _pwdDSEntry.fakeInitModel (_agreement.getBindCredentials());
            _bindDSEntry.fakeInitModel (_agreement.getBindDN());
            _origPwd    = _agreement.getBindCredentials();
            _origBindDN = _agreement.getBindDN();
        }
        if (_agreement.getUseSSL()) {
            _sslDSEntry.fakeInitModel ("on");
			_sslEncrypt.setSelected(true); 
            _portAttr = ATTR_SSL_PORT;
			
            if (_agreement.getUseSSLAuth()) {
                //SSL client Auth
                enableSimpleAuth (false);
                _sslAuth.setEnabled(true);
                _sslAuthDSEntry.fakeInitModel ("on");
				_simpAuthDSEntry.fakeInitModel ("off");
				
            } else {
                //Simple Auth with SSL
                _sslAuth.setEnabled(true);
                _simpAuth.setSelected(true);
                _simpAuthDSEntry.fakeInitModel ("on");
                enableSimpleAuth (true);
            }
        } else {
            _portAttr = ATTR_PORT;
            setSimpAuth();
        }
        if(_agreement.getAgreementType() == ReplicationAgreement.AGREEMENT_TYPE_AD){
            if(((ActiveDirectoryAgreement)_agreement).getNewWinUserSync().compareTo("on") == 0){
                _newWinUserSyncCB.setSelected(true);
                _newUserDSEntry.fakeInitModel("on");
            }else{
                _newWinUserSyncCB.setSelected(false);
                _newUserDSEntry.fakeInitModel("off");
            }
            if(((ActiveDirectoryAgreement)_agreement).getNewWinGroupSync().compareTo("on") == 0){
                _newWinGroupSyncCB.setSelected(true);
                _newGroupDSEntry.fakeInitModel("on");
            }else{
                _newWinGroupSyncCB.setSelected(false);
                _newGroupDSEntry.fakeInitModel("off");
            }
        }
    }

    /**
     * Set simple auth selection
     */
    private void setSimpAuth() {
        //Simple Auth no SSL
        _sslDSEntry.fakeInitModel ("off");
        _sslAuth.setEnabled(false);
        _simpAuthDSEntry.fakeInitModel ("on");
    }

    private void enableSimpleAuth (boolean enable){
        _bind.setEnabled(enable);
        _bind.repaint(1);
        _bindText.setEnabled(enable);
        _bindText.repaint(1);
        _pwd.setEnabled(enable);
        _pwd.repaint(1);    
        _pwdText.setEnabled(enable);
	if (enable)
	    _pwdText.setBackground(_bindText.getBackground());
	else
	    _pwdText.setBackground(getBackground());         
    }

	/**
	  * Method used to know wether the password has been modified or not.
	  *
	  *@return boolean telling if the password has been modified or not
	  */
	public boolean isPwdDirty() {
		return _pwdDSEntry.getDirty();
	}

    class bindDSEntry extends DSEntryTextStrict
    {
        bindDSEntry(String model, JComponent view1, JComponent view2) {
            super (model, view1, view2);
        }

        public int validate (){
            JTextField tf = (JTextField) getView (0);
            String     dn = tf.getText ();

            /* disabled field is always valid */
            if (!tf.isEnabled ())
                return 0;

            if (!dn.equals ("") && DSUtil.isValidDN (dn))
                return 0;
            else
                return 1;
        }
    }

    /*==========================================================
     * variables
     *==========================================================*/
    private ConsoleInfo _serverInfo;
    private ConsoleInfo _consoleInfo;
    private AgreementInfoPanel _monitor;
    private Hashtable _dsInstances;
    private JTabbedPane _tabbedPanel;;
    private Hashtable _baseSuffixes = null;
    
    private boolean refreshed = false;

    JTextField _bindText;
    private JCheckBox _sslEncrypt, _newWinUserSyncCB, _newWinGroupSyncCB;
    private JRadioButton _simpAuth, _sslAuth;
    JPasswordField _pwdText;
    private JLabel _bind, _pwd, _newWinUserSync, _newWinGroupSync;
    private JFrame _frame;
    private ImageIcon _icon;

    private bindDSEntry        _bindDSEntry;
    private DSEntryTextStrict  _pwdDSEntry;
    private DSEntryBoolean     _newUserDSEntry;
    private DSEntryBoolean     _newGroupDSEntry;
    private DSEntryBoolean     _sslDSEntry;
    private DSEntryBoolean     _sslAuthDSEntry;
    private DSEntryBoolean     _simpAuthDSEntry;

    private String _origBindDN;
    private String _origPwd; 

    private String _portAttr = ATTR_PORT;

    private static final int CONTINUE = 0;
    private static final int PROMPT = 1;
    private static final int SKIP =3;

    //get resource bundle
    public static ResourceSet _resource =
	new ResourceSet("com.netscape.admin.dirserv.panel.replication.replication");
    private static final String _section = "replication-destination-dialog";
    private static final String DSClass = "nsDirectoryServer";
    private static final String NSClass = "netscapeServer";
    private static final String ATTR_PORT = "nsserverport";
    private static final String ATTR_SSL_PORT = "nssecureserverport";
}
